# -*- coding: utf-8 -*
# Copyright (C) Sebastien Tricaud 2012-2020

import os
import sys
import platform

from ctypes import *

from pyfaup.bind import *

rundir = os.path.dirname(os.path.abspath(__file__))
os.environ["FAUP_DATA_DIR"] = rundir 

is_32bits = (sys.maxsize <= 2**32)
if is_32bits:
        raise ImportError("32 bits architectures not supported")

system = platform.system()

LOAD_LIB=""

if system == "Linux":
        LOAD_LIB=rundir + "/Linux/x86_64/libfaupl.so"
if system == "Darwin":
        LOAD_LIB=rundir + "/Darwin/x86_64/libfaupl.dylib"

#print(LOAD_LIB)
        
bind.library = cdll.LoadLibrary(LOAD_LIB)



"""Version and details for pyfaup"""

__description__ = "Python bindings for the faup library"
__url__ = "https://www.github.com/stricaud/faup"
__version__ = "1.1"
__author__ = "Sebastien Tricaud"
__author_email__ = "sebastien@honeynet.org"
__license__ = "MIT"
__maintainer__ = __author__
__maintainer_email__ = __author_email__



# -*- coding: utf-8 -*
# Copyright (C) Sebastien Tricaud 2012

from ctypes import *
from .bind import *

faup_get_version = bind("faup_get_version", c_char_p)
faup_init = bind("faup_init", c_void_p, c_void_p)
faup_decode = bind("faup_decode", c_char_p, c_void_p, c_char_p, c_size_t)
faup_terminate = bind("faup_terminate", None, c_void_p)

faup_get_scheme_pos = bind("faup_get_scheme_pos", c_int, c_void_p)
faup_get_scheme_size = bind("faup_get_scheme_size", c_uint, c_void_p)
faup_get_credential_pos = bind("faup_get_credential_pos", c_int, c_void_p)
faup_get_credential_size = bind("faup_get_credential_size", c_uint, c_void_p)
faup_get_subdomain_pos = bind("faup_get_subdomain_pos", c_int, c_void_p)
faup_get_subdomain_size = bind("faup_get_subdomain_size", c_uint, c_void_p)
faup_get_domain_pos = bind("faup_get_domain_pos", c_int, c_void_p)
faup_get_domain_size = bind("faup_get_domain_size", c_uint, c_void_p)
faup_get_domain_without_tld_pos = bind("faup_get_domain_without_tld_pos", c_int, c_void_p)
faup_get_domain_without_tld_size = bind("faup_get_domain_without_tld_size", c_uint, c_void_p)
faup_get_host_pos = bind("faup_get_host_pos", c_int, c_void_p)
faup_get_host_size = bind("faup_get_host_size", c_uint, c_void_p)
faup_get_tld_pos = bind("faup_get_tld_pos", c_int, c_void_p)
faup_get_tld_size = bind("faup_get_tld_size", c_uint, c_void_p)
faup_get_port_pos = bind("faup_get_port_pos", c_int, c_void_p)
faup_get_port_size = bind("faup_get_port_size", c_uint, c_void_p)
faup_get_resource_path_pos = bind("faup_get_resource_path_pos", c_int, c_void_p)
faup_get_resource_path_size = bind("faup_get_resource_path_size", c_uint, c_void_p)
faup_get_query_string_pos = bind("faup_get_query_string_pos", c_int, c_void_p)
faup_get_query_string_size = bind("faup_get_query_string_size", c_uint, c_void_p)
faup_get_fragment_pos = bind("faup_get_fragment_pos", c_int, c_void_p)
faup_get_fragment_size = bind("faup_get_fragment_size", c_uint, c_void_p)

faup_options_new = bind("faup_options_new", c_void_p)
faup_options_free = bind("faup_options_free", None, c_void_p)



# -*- coding: utf-8 -*
# Copyright (C) Sebastien Tricaud 2012

from ctypes import *

def bind(cfunction, restype, *argtypes):
	func = getattr(bind.library, cfunction)
	func.restype = restype
	func.argtypes = argtypes
	return func




import sys
import chardet

from .functions import *

class UrlNotDecoded(Exception):
    pass

class Faup(object):
    """
    Faup Python Library 
    """
    def __init__(self):
        self.options = faup_options_new()
        self.fh = faup_init(self.options)
        self.decoded = False
        self.retval = {}
        self.python_ver = sys.version_info.major

    def __del__(self):
        faup_terminate(self.fh)
        faup_options_free(self.options)

    def decode_str(self, string):        
        if self.python_ver >= 3:
            if string:
                return string.decode("utf-8")
        
        return string
        
    def decode(self, url):
        """
        This function creates a dict of all the url fields.

        :param url: The URL to normalize
        """
        self._url = None

        if self.python_ver >= 3:
            self._url = bytes(url.encode('utf-8'))
        else:
            self._url = bytes(url)
        
        self._url = faup_decode(self.fh, self._url, len(self._url))
        self.decoded = True
        self.retval = {}
    
    @property    
    def url(self):
        return self._url
        
    def get_version(self):
        return faup_get_version()

    def _get_param_from_pos_and_size(self, pos, size):
        if pos < 0:
            return None

        else:
            return self._url[pos:(pos+size)]

    def get_scheme(self):
        """
        Get the scheme of the url given in the decode function

        :returns: The URL scheme
        """
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_scheme_pos(self.fh)
        size = faup_get_scheme_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_credential(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_credential_pos(self.fh)
        size = faup_get_credential_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_subdomain(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")


        pos = faup_get_subdomain_pos(self.fh)
        size = faup_get_subdomain_size(self.fh)
        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_domain(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_domain_pos(self.fh)
        size = faup_get_domain_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_domain_without_tld(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_domain_without_tld_pos(self.fh)
        size = faup_get_domain_without_tld_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))
        
    def get_host(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_host_pos(self.fh)
        size = faup_get_host_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))
        
    def get_tld(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_tld_pos(self.fh)
        size = faup_get_tld_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))
            
    def get_port(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_port_pos(self.fh)
        size = faup_get_port_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_resource_path(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_resource_path_pos(self.fh)
        size = faup_get_resource_path_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_query_string(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_query_string_pos(self.fh)
        size = faup_get_query_string_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get_fragment(self):
        if not self.decoded:
            raise UrlNotDecoded("You must call faup.decode() first")

        pos = faup_get_fragment_pos(self.fh)
        size = faup_get_fragment_size(self.fh)

        return self.decode_str(self._get_param_from_pos_and_size(pos, size))

    def get(self):        
        self.retval["scheme"] = self.get_scheme()
        self.retval["tld"] = self.get_tld()    
        self.retval["domain"] = self.get_domain()
        self.retval["domain_without_tld"] = self.get_domain_without_tld()
        self.retval["subdomain"] = self.get_subdomain()    
        self.retval["host"] = self.get_host()
        self.retval["port"] = self.get_port()
        self.retval["resource_path"] = self.get_resource_path()
        self.retval["query_string"] = self.get_query_string()
        self.retval["fragment"] = self.get_fragment()
        self.retval["url"] = self.url
        return self.retval



#!/usr/bin/python
from setuptools import setup, find_packages
#from distutils.core import setup
import os
import textwrap

current_dir = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(current_dir, "pyfaup", "__version__.py"), "r") as fp:
          exec(fp.read(), about)

setup(
    name='pyfaup',
    version = about["__version__"],
    description= about["__description__"],
    author= about["__author__"],
    author_email= about["__author_email__"],
    long_description=textwrap.dedent(open("README.rst", "r").read()),
    long_description_content_type="text/x-rst",
    maintainer=about["__maintainer__"],
    maintainer_email=about["__maintainer_email__"],
    url=about["__url__"],
    license=about["__license__"],
    packages = ["pyfaup"],
    package_dir = {"pyfaup": "pyfaup/"},
    package_data = {'pyfaup': ['Linux/x86_64/libfaupl.so','Darwin/x86_64/libfaupl.dylib','mozilla.tlds']},
)
